##############################################################################
#
#	resource.pm	-	Resource class
#	
#	This file is part of the EETCO tool written by 
#	Damien Hardy, Isidoros Sideris and Yiannakis Sazeides
#	University of Cyprus
#	The research leading to this tool is supported by the European Commission
#	FP7 project ”Energy-conscious 3D Server-on-Chip for Green Cloud
#	Services (Project No:247779 ”EuroCloud”)”
#
#	This source file is distributed "as is" in the hope that it will be useful.
#	The tool comes with no warranty, and no author or distributor accepts any
#	responsability for the consequences of its use.
#
#	Everyone is granted permission to copy, modify and redistribute
#	this tool set under the following conditions:
#
#		This source code is distributed for non-commercial use only.
#		Please contact the maintainer for restrictions applying to
#		commercial use.
#
#		Permission is granted to anyone to make or distribute copies
#		of this source code, either as received or modified, in any
#		medium, provided that all copyright notices, permission and
#		nonwarranty notices are preserved, and that the distributor
#		grants the recipient permission for further redistribution as
#		permitted by this document.
#
#		Permission is granted to distribute this file in compiled
#		or executable form under the same conditions that apply for
#		source code, provided that either:
#
#		A.	it is accompanied by the corresponding machine-readable
#			source code,
#		B.	it is accompanied by a written offer, with no time limit,
#			to give anyone a machine-readable copy of the corresponding
#			source code in return for reimbursement of the cost of
#			distribution.  This written offer must permit verbatim
#			duplication by anyone, or
#		C.	it is distributed by someone who received only the
#			executable form, and is accompanied by a copy of the
#			written offer of source code that they received concurrently.
#
#	In other words, you are welcome to use, share and improve this
#	source file.  You are forbidden to forbid anyone else to use, share
#	and improve what you give them.
#
#	The research leading to this tool is supported by the European Commission 
#	FP7 project "Energy-conscious 3D Server-on-Chip for Green Cloud Services 
#	(Project No:247779 "EuroCloud")".
#
#	Tool: http://www.cs.ucy.ac.cy/carch/xi/eetco.php
#	Project: http://www.eurocloudserver.com/
#	European Commission FP7: http://cordis.europa.eu/fp7/home_en.html
# 
##############################################################################

package resource;

use strict;
use warnings;

use dc;

use POSIX qw/floor/;
use POSIX qw/ceil/;


#list of parameters which have to be present in a datacenter configuration file
@resource::list_parameters_name=("resource_name","n_srvmodulesreq","u","coef_maintenance","k_bladesperrack","k_modulesperblade","height","width","depth","minimumdistance","useddistance","srvdepr","mttr","nb_time_step","SPUE","k_chipspersrvmodule","c_system","p_system","p_system_idle","n_dram","c_dram","p_dram","p_dram_idle","mttf_dram","n_disk","c_disk","p_disk","p_disk_idle","mttf_disk","n_switchport","c_switchport","p_switchport","p_switchport_idle","mttf_switchport","c_other","p_other","p_other_idle","c_chip","pvf","p_chip","p_chip_idle","chip_mttf","c_networkperrack","networkdepr","network_peak_powerperrack","chip_perf");
#condition they have to meet
%resource::input_parameters_check_values=("resource_name","*", "n_srvmodulesreq","<=" , "u","<" , "coef_maintenance","<" , "k_bladesperrack","<" , "k_modulesperblade","<" ,"height","<" ,"width","<" ,"depth","<" ,"minimumdistance","<" ,"useddistance","<" ,"srvdepr","<=" ,"mttr","<=" ,"nb_time_step","<=" ,"SPUE","<" ,"k_chipspersrvmodule","<=" ,"c_system","<" ,"p_system","<" ,"p_system_idle","<" ,"n_dram","<" ,"c_dram","<" ,"p_dram","<" ,"p_dram_idle","<" ,"mttf_dram","<=" ,"n_disk","<" ,"c_disk","<" ,"p_disk","<" ,"p_disk_idle","<" ,"mttf_disk","<=" ,"n_switchport","<" ,"c_switchport","<" ,"p_switchport","<" ,"p_switchport_idle","<" ,"mttf_switchport","<=" ,"c_other","<" ,"p_other","<" ,"p_other_idle","<" ,"c_chip","<" ,"pvf","<" ,"p_chip","<" ,"p_chip_idle","<" ,"chip_mttf","<=","c_networkperrack","<","networkdepr","<=","network_peak_powerperrack","<","chip_perf","<");


################################################
#
#	constructor & initialization
#	(parser of the input file)
#
################################################
sub new($$)
{
	my ($class, $filename, $dc)= @_;
	my $self={};
	bless ($self , $class);
	
	$self->{filename}=$filename;
	$self->{dc}=$dc;
	
	if(!open FILE,"<$filename")
	{
		die "cannot open file: $filename\n";
	}
	while(defined(my $line=<FILE>))
	{
		if($line =~ /^\s*(\w+)\s+(\d+(?:.\d+)?(?:e[+|-]?\d+)?)/)
		{
			my $variable=$1;
			my $value=$2;
			
			$self->{"$variable"}=$2;
		}
		elsif($line =~ /^\s*resource_name\s+(\w+)/)
		{
			$self->{resource_name}=$1;
		}
	}
	close FILE;

	my $error=0;	
	###check if all the parameters have been correctly defined
	foreach my $pname (@resource::list_parameters_name)
	{
		if(!defined($self->{"$pname"}))
		{
			print("$pname not defined\n");
			$error=1;
		}
		else
		{
			if($resource::input_parameters_check_values{"$pname"} eq "<=")
			{
				if($self->{"$pname"} <= 0)
				{
					$error=1;
					print("$pname not correctly defined\n");
				}
			}

			if($resource::input_parameters_check_values{"$pname"} eq "<")
			{
				if($self->{"$pname"} < 0)
				{
					$error=1;
					print("$pname not correctly defined\n");
				}
			}
		}
	}
	
	if($error==1)
	{
		die "Interrupt:parameters are not defined in $filename\n";
	}
	
	if($self->{coef_maintenance} >1)
	{
		print("coef_maintenance not correctly defined\n");
		die "Interrupt:parameters are not defined in $filename\n";
	}
	if($self->{pvf} >1)
	{
		print("pvf not correctly defined\n");
		die "Interrupt:parameters are not defined in $filename\n";
	}
	##end check

	return $self;
}


################################################
#
#	print input param
#
################################################
sub print_param()
{
	my $self=shift;
	
	foreach my $pname ( @resource::list_parameters_name )
	{
		my $val= $self->{"$pname"};
		print("$pname = $val\n");
	}
	
	#optional parameters
	if(defined($self->{"mttf_coef"})){print("mttf_coef = $self->{mttf_coef}\n");}
	
	
	print("-----------------------\n");
}



################################################
#
#	accessors
#		get (variable name)
#		set (variable name,value)
#
################################################
sub get($)
{
	my ($class, $pname)= @_;
	my $self=shift;
	return $self->{"$pname"};
}

sub set($$)
{
	my ($class, $pname,$val)= @_;
	my $self=shift;
	$self->{"$pname"}=$val;
}


###########################################
#
#	check_validity Function
#
# To ensure that the mttf of a server is higher
# than its depreciation
#
###########################################
sub check_validity()
{
	my $self=shift;
	my $mttf_all= 1/ ( ($self->{k_chipspersrvmodule}/$self->{chip_mttf})+($self->{n_dram}/$self->{mttf_dram})+($self->{n_disk}/$self->{mttf_disk})+($self->{n_switchport}/$self->{mttf_switchport}) );
	
	if( $mttf_all<=$self->{srvdepr} ){print "Error: MTTF of srv smaller than srvdepr\n";exit(0);}
}


###########################################
#
#	Spares Estimation Function
#
###########################################

sub sparesEstimation()
{
	my $self=shift;
	
	my $mttf_all= 1/ ( ($self->{k_chipspersrvmodule}/$self->{chip_mttf})+($self->{n_dram}/$self->{mttf_dram})+($self->{n_disk}/$self->{mttf_disk})+($self->{n_switchport}/$self->{mttf_switchport}) );
	
	my $mttf=$mttf_all *365.0; # * by 365 to have mttf in days
	
	#optional parameter 'mttf_coef' to adjust the mttf in case the ambient temperature increases
	if(defined($self->{mttf_coef}))
	{
		$mttf=$mttf*$self->{mttf_coef};
	}
	#end optional parameter
	
	my $t=$self->{srvdepr} * 365;		# server depreciation in day


	my $k=$self->{nb_time_step};
	
	#check $k value
	if($k>$t)
	{
		print("warning: nb_time_step > srvdepr. Assumed in the computation: nb_time_step == srvdepr in days\n");
		$k=$t;
	}
	
	my $tau=$t/$k;
	
	my $N=$self->{n_srvmodulesreq};
	
	my $pvf0=$self->{pvf};
	my $perf0=1-$pvf0;
	
	my $hs=ceil($N/$perf0-$N); #hot spares


	my $Nservermodules=$N+$hs;
	########
	
	#########################################################	
	#cold spares estimation
	my @d;
	for(my $i=0;$i<=$k;$i++)
	{
		$d[$i]=exp(-$i*$tau/$mttf);
	}

	my @cs; #cold spares
	$cs[0]=0;# at time 0 cs=0
	$cs[1]=ceil( ($Nservermodules/$d[1]) - $Nservermodules );
	
	for(my $i=2;$i<=$k;$i++)
	{
		my $sum=0;
		for(my $j=1;$j<$i;$j++)
		{
			$sum+= ($cs[$i-$j]-$cs[$i-$j-1])/$d[$j];
		}
		
		$cs[$i]=ceil ( $sum + ($Nservermodules-$cs[$i-1])/$d[$i] - $Nservermodules );
		
	}
	
	##store results
	$self->{hotspares}=$hs;
	$self->{coldspares}=$cs[$k];
}


###########################################
#
#	c_server Estimation Function
#
###########################################
sub c_serverEstimation()
{
	my $self=shift;
	
	my $c_srv_other_modulecomponents=$self->{c_other} + $self->{c_dram}*$self->{n_dram} + $self->{c_disk}*$self->{n_disk} + $self->{c_switchport}*$self->{n_switchport}  + $self->{c_system} ;
	my $c_srvmodule=$self->{c_chip}*$self->{k_chipspersrvmodule} + $c_srv_other_modulecomponents;
	
	my $n_srvmodules=$self->{n_srvmodulesreq}+$self->{hotspares};

	my $c_server=($n_srvmodules*$c_srvmodule)/($self->{srvdepr}*12);

	
	#cost of money
	my $dc=$self->{dc};
	if(defined($dc->get("cost_of_money")))
	{
		my $com=$dc->get("cost_of_money");
		if($com>0)
		{
			my $c_server_total = $n_srvmodules*$c_srvmodule;
			$c_server = ($c_server_total * $com/12) / (1 - (1+$com/12)**(-$self->{srvdepr}*12));
		}
	}
	#
	
	##store results
	$self->{c_srvmodule}=$c_srvmodule;
	$self->{n_srvmodules}=$n_srvmodules;
	$self->{c_server}=$c_server;
	
	return $c_server;
}


###########################################
#
#	c_power Estimation Function
#
###########################################
sub c_powerEstimation()
{
	my $self=shift;
	
	my $p_srv_other_modulecomponents =  $self->{p_other} + $self->{p_dram}*$self->{n_dram} + $self->{p_disk}*$self->{n_disk} + $self->{p_switchport}*$self->{n_switchport} + $self->{p_system} ;
	my $p_srv_other_modulecomponents_idle = $self->{p_other_idle} + $self->{p_dram_idle}*$self->{n_dram} + $self->{p_disk_idle}*$self->{n_disk} + $self->{p_switchport_idle}*$self->{n_switchport} + $self->{p_system_idle} ;

	my $p_total_peak = $self->{n_srvmodules}*($self->{k_chipspersrvmodule}*$self->{p_chip}+$p_srv_other_modulecomponents) ;
	my $p_total_idle = $self->{n_srvmodules}*($self->{k_chipspersrvmodule}*$self->{p_chip_idle}+$p_srv_other_modulecomponents_idle) ;
	my $p_total_avg = $self->{u}*$p_total_peak+(1-$self->{u})*$p_total_idle;

	$p_total_avg*=$self->{SPUE};
	$p_total_peak*=$self->{SPUE};
	
	#add network power consumption
	my $k_modulesperrack=$self->{k_bladesperrack}*$self->{k_modulesperblade};
	my $n_racks=ceil($self->{n_srvmodules}/$k_modulesperrack);
	
	$p_total_peak += $self->{network_peak_powerperrack} * $n_racks ;
	$p_total_avg += $self->{network_peak_powerperrack} * $n_racks ;
	#
	
	#compute peak power per server and rack
	my $p_peak_per_server = ($self->{k_chipspersrvmodule}*$self->{p_chip}+$p_srv_other_modulecomponents)*$self->{SPUE};
	my $p_peak_per_rack = $k_modulesperrack*$p_peak_per_server + $self->{network_peak_powerperrack};
	#
	
	my $dc=$self->{dc};
	
	my $c_power= ( ( $dc->get("PUE")*$p_total_avg* 30*24 ) / 1000  ) *  $dc->get("c_electperKWH") ; #30*24/1000 -> to translate Watts in KWh per month

	#optional parameter 'c_elecpeak' to consider the peak power consumption to estimate the power cost
	if(defined($dc->get("c_elecpeak")))
	{
		if($dc->get("c_elecpeak")==1)
		{
			$c_power= ( ( $dc->get("PUE")*$p_total_peak* 30*24 ) / 1000  ) *  $dc->get("c_electperKWH") ;
		}
	}
	#end optional parameter
	
	##store results
	$self->{p_peak_per_server}=$p_peak_per_server;
	$self->{p_peak_per_rack}=$p_peak_per_rack;
	
	$self->{p_total_peak}=$p_total_peak;
	$self->{p_total_avg}=$p_total_avg;
	$self->{c_power}=$c_power;
	
	return $c_power;
}



###########################################
#
#	c_acquisition Estimation Function
#
###########################################
sub c_acquisitionEstimation()
{
	my $self=shift;
	
	my $dc=$self->{dc};
	
	my $k_modulesperrack=$self->{k_bladesperrack}*$self->{k_modulesperblade};
	my $a_perrack=$self->{width}*($self->{depth}+$self->{useddistance});

	my $n_racks=ceil($self->{n_srvmodules}/$k_modulesperrack); #number of racks
	my $a_it=$n_racks*$a_perrack; #IT area
	
	my $a_dc=$a_it*$dc->get("a_coolingarea"); #extra area for cooling
	my $c_building_acquisition=$a_dc*$dc->get("c_buildingpersqm"); #building cost

	my $c_cooling_acquisition=$dc->get("c_coolpowerdeplperW")*$self->{p_total_peak};

	my $c_acquisition=($c_building_acquisition+$c_cooling_acquisition)/($dc->get("dcdepr")*12);

	#cost of money
	if(defined($dc->get("cost_of_money")))
	{
		my $com=$dc->get("cost_of_money");
		if($com>0)
		{
			my $c_acquisition_total = $c_building_acquisition+$c_cooling_acquisition ;
			$c_acquisition = ($c_acquisition_total * $com/12) / (1 - (1+$com/12)**(-$dc->get("dcdepr")*12));
		}
	}
	#
	
	
	##store results
	$self->{a_perrack}=$a_perrack;
	$self->{a_dc}=$a_dc;
	$self->{n_racks}=$n_racks;
	$self->{c_acquisition}=$c_acquisition;
	
	return $c_acquisition;
}

###########################################
#
#	c_maintenance Estimation Function
#
###########################################
sub c_maintenanceEstimation()
{
	my $self=shift;
	
	my $dc=$self->{dc};

	my $c_maintenance=($self->{coldspares}*$self->{c_srvmodule}*$self->{coef_maintenance})/($self->{srvdepr}*12)+ $self->{n_racks} * $dc->get("salary_perrackpermonth");

	##store results
	$self->{c_maintenance}=$c_maintenance;
	
	return $c_maintenance;
}


###########################################
#
#	c_network Estimation Function
#
###########################################
sub c_networkEstimation()
{
	my $self=shift;
	
	my $k_modulesperrack=$self->{k_bladesperrack}*$self->{k_modulesperblade};
	my $n_racks=ceil($self->{n_srvmodules}/$k_modulesperrack); #number of racks
	my $c_network = ($n_racks * $self->{c_networkperrack}) / ($self->{networkdepr} * 12);
	
	#cost of money
	my $dc=$self->{dc};
	if(defined($dc->get("cost_of_money")))
	{
		my $com=$dc->get("cost_of_money");
		if($com>0)
		{
			my $c_network_total = $n_racks * $self->{c_networkperrack} ;
			$c_network = ($c_network_total * $com/12) / (1 - (1+$com/12)**(-$self->{networkdepr}*12));
		}
	}
	#
	
	##store results
	$self->{c_network}=$c_network;
	
	return $c_network;
}



###########################################
#
#	availability Estimation Function
#
###########################################
sub availabilityEstimation()
{
	my $self=shift;
	
	my $availability = 1 / ( 1 + ($self->{coldspares} * $self->{mttr}/(24*365))/ ( $self->{srvdepr} * ($self->{hotspares} + $self->{n_srvmodulesreq}) ) );
	
	##store result
	$self->{availability}=$availability;
}


###########################################
#
#	print results Function
#
###########################################
sub print_results()
{
	my $self=shift;
	
	my $dc=$self->{dc};
	
	my $tco=myround($self->{c_acquisition} + $self->{c_power} + $self->{c_server} + $self->{c_maintenance} + $self->{c_network});	
	my $c_acquisition=myround($self->{c_acquisition});
	my $c_power=myround($self->{c_power});
	my $c_server=myround($self->{c_server});
	my $c_maintenance=myround($self->{c_maintenance});
	my $c_network=myround($self->{c_network});
	
	print("Resource: $self->{resource_name}\n");
	
	print ("\ntco: $tco \$/month \n");
	
	print("\tc_infrastructure: \t$c_acquisition \$/month\n");
	print("\tc_server: \t\t$c_server \$/month\n");
	print("\tc_power: \t\t$c_power \$/month\n");
	print("\tc_network: \t\t$c_network \$/month\n");
	print("\tc_maintenance: \t\t$c_maintenance \$/month\n\n");
	
	print("n_racks: \t$self->{n_racks}\n");
	print("a_perrack: \t$self->{a_perrack} m^2\n");
	
	my $p_peakperserver = myround($self->{p_peak_per_server});
	#print("p_peakperserver: $p_peakperserver W\n");
	
	my $p_peakperrack = myround($self->{p_peak_per_rack});
	print("p_peakperrack:\t$p_peakperrack W\n");
	
	my $a_dc=myround($self->{a_dc});
	print("a_dc: \t\t$a_dc m^2\n");
	
	print("n_srvmodules: \t$self->{n_srvmodules}\n");	
	#print("n_hotsp: \t$self->{hotspares}\n");
	print("n_coldsp: \t$self->{coldspares}\n");
	print("c_srvmodule: \t$self->{c_srvmodule} \$\n");
	print("availability: \t$self->{availability}\n\n");
	
	my $p_total_avg=myround($self->{p_total_avg}*$dc->get("PUE"));
	my $p_total_peak=myround($self->{p_total_peak}*$dc->get("PUE"));
	print("p_total_avg: \t$p_total_avg W\n");
	print("p_total_peak:\t$p_total_peak W\n\n");
	
	#perf
	my $perf_total=$self->{n_srvmodules}*$self->{k_chipspersrvmodule}*$self->{chip_perf};
	$perf_total*=(1 - $self->{pvf});
	
	my $perf_dividedby_tco=myround(1000 * $perf_total/$tco);
	my $perf_dividedby_watt=myround(1000 *$perf_total/$p_total_peak);
	$perf_total=myround($perf_total);
	
	print("Performance: \t\t $perf_total\n");
	print("PerformanceOverTCO:\t $perf_dividedby_tco x1000\n");
	print("PerformanceOverWatt:\t $perf_dividedby_watt x1000\n\n");
	
	
	print("-----------------------\n");
	
}




###########################################
#
#	round function
#
###########################################
sub myround#($)
{
	my $res = $_[0];
	if($res <0){$res = int(100*$res - 0.5)/100 ;}
	else{$res = int(100*$res + 0.5)/100 ;}
	return $res;
}



####################################################
####################################################
####################################################
####################################################





1;




