#!/usr/bin/perl -w

######################################################
#
#	2D wrapper exploration
#
#	The 2D wrapper has the ability to produce a 2D exploration
#	by doing a sweep on the x-axis
#	and show the normalized value of an output parameter on the y-axis
#
#	The results are printed in a file <output_file_name>.txt
#	and a graph can be plot in a file <output_file_name>.eps
#
#	How to use it:
#	./2Dexploration.pl	<dc config file>								datacenter configuration file
#						<resource config file>							resource configuration file
#						<output_file_name>								filename of the output file without extension (the extensions are .txt and .eps)
#						<output_result[:reference value]>				output value name to use
#																		optional: the reference value is optional if you want provide a value comming from another configuration
#																		in that case you can put 1 to obtain absolute values
#						<x_parameter_name>								parameter x name 
#						<x_parameter_value_ref>							reference value of x 
#						<x_parameter_range_min>							minimale value of the x range
#						<x_parameter_range_max>							maximale value of the x range
#						<x_parameter_range_step>						number of steps in the x range
#						<x_parameter_range_type: absolute/relative>		the range can be absolute or relative
#
#						[xaxis:name]									optional to put a name 'name' for the x-axis of the graph (by default <x_parameter_name> is used)
#						[yaxis:name]									optional to put a name 'name' for the y-axis of the graph (by default <y_parameter_name> is used)
#						[curve:name]									optional to put a name 'name' for the curve of the graph (by default <output_result> is used)
#						[nognuplot]										optional: put "nognuplot" if you do not want to get the graph
#
#																
#
#  Example of command:
#  ./2Dexploration.pl template/dc_template.params template/r_template.params tco tco pvf 0 0 0.1 0.01 absolute xaxis:PVF yaxis:"Normalized TCO" curve:"Normalized TCO"
#
######################################################


use strict;

use POSIX qw/ceil/;

use File::Copy;

use lib "../lib/"; 
use helper;


######################################################
#
# Get input parameters
#
######################################################

my $argv_size = scalar @ARGV;
if ($argv_size < 10)
{
	print "usage: ./2Dexploration.pl <dc config file> <resource config file> <output_file_name> <output_result[:reference value]> <x_parameter_name> <x_parameter_value_ref> <x_parameter_range_min> <x_parameter_range_max> <x_parameter_range_step> <x_parameter_range_type: absolute/relative>  [xaxis:name] [yaxis:name] [curve:name] [nognuplot]\n";
	exit(0);
}

my $dc_filename=$ARGV[0];
my $resource_filename=$ARGV[1];

my $filename= $ARGV[2];

my $output_name=$ARGV[3];

my $cst_defined=0;
my $ref=0;
my @tmp=split(':' ,$output_name);
my $tmp_size=scalar @tmp;
if($tmp_size==2)
{
	$output_name=$tmp[0];
	$ref=$tmp[1];
	$cst_defined=1;
}
elsif($tmp_size==1){;}
else{print("error: $output_name not correctly defined\n");exit(0);}

my $x_name=$ARGV[4];
my $x_ref=$ARGV[5];
my $x_min=$ARGV[6];
my $x_max=$ARGV[7];
my $x_step=$ARGV[8];
my $x_relative;
if($ARGV[9] eq "relative")
{
	$x_relative=1;
}
elsif($ARGV[9] eq "absolute")
{
	$x_relative=0;
}
else{print("error: x_parameter_range_type has to be relative or absolute\n");exit(0);}

my $x_axis_name=$x_name;
my $y_axis_name=$output_name;
my $curve_name=$output_name;
my $gnuplot=1;
for(my $i=10;$i<14;$i++)
{
	if(defined($ARGV[$i]))
	{
		my $arg_tmp=$ARGV[$i];
		
		my @list=split(':',$arg_tmp);
		
		if( $list[0] eq "nognuplot")
		{
			$gnuplot=0;
		}
		elsif($list[0] eq "xaxis")
		{
			if(scalar(@list)!=2){print ("Error: parameter $arg_tmp not correctly defined\n");exit(0);}
			$x_axis_name=$list[1];
		}
		elsif($list[0] eq "yaxis")
		{
			if(scalar(@list)!=2){print ("Error: parameter $arg_tmp not correctly defined\n");exit(0);}
			$y_axis_name=$list[1];
		}
		elsif($list[0] eq "curve")
		{
			if(scalar(@list)!=2){print ("Error: parameter $arg_tmp not correctly defined\n");exit(0);}
			$curve_name=$list[1];
		}
		else{print ("Error: parameter $arg_tmp not correctly defined\n");exit(0);}
	}
	else{last;}
}



######################################################
#
# General Initialization
#
######################################################

#create the helper
my $helper = helper->new("");

#get the parameters from files
my $ptr_dc=$helper->parserDCFile("$dc_filename");
my %dc=%{$ptr_dc};

my $ptr_resource=$helper->parserResourceFile("$resource_filename");
my %resource=%{$ptr_resource};

#check that the inputs are correct
if($helper->checkInputParameterName($x_name)==-1){print("Execution stopped\n");exit(0);}
#check that the output name is correct
if($helper->checkOutputValueName($output_name)==-1){print("Execution stopped\n");exit(0); }
if($helper->checkOutputValueName($output_name)==1)
{
	if(defined($dc{report_resources}))
	{
		if($dc{report_resources}!=1)
		{
			print("Error: The parameter 'report_resources' in the DC file must be set to 1 to get results\n");
			exit(0);
		}
	}
}



#to get the current dir
my $currentdir=`pwd`;
chomp $currentdir;

#open outputfile
open(OUTPUTFILE,">$currentdir/output/$filename.txt") || die ("Error when opening $currentdir/output/$filename.txt") ;


######################################################
#
# Specific Initialization
#
######################################################

	####################################################
	#get the reference value 
	#(if needed i.e. $cst_defined==0)
	####################################################

	if($cst_defined==0)
	{
		#generate the current configuration file
		$dc{"$x_name"}=$x_ref;
		$helper->createParameterFile("$currentdir/dc.params",\%dc);

		$resource{"$x_name"}=$x_ref;
		$helper->createParameterFile("$currentdir/s.params",\%resource);

		#call the eetco kernel and keep the output
		$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/ref");

		#get the result value
		my $ptr_res=$helper->parserResultKernel("$currentdir/output/ref");
		my %res=%{$ptr_res};
		$ref=$res{"$output_name"};
	}

####################################################
# main exploration loop
####################################################

#exploration loop of x
for(my $x=$x_min;$x<=$x_max;$x+=$x_step)
{
	my $x_value=$x;	
	if($x_relative==1)
	{
		$x_value=$x * $x_ref;
	}
		
		#generate the current configuration file
		$dc{"$x_name"}=$x_value;
		$helper->createParameterFile("$currentdir/dc.params",\%dc);
		
		$resource{"$x_name"}=$x_value;
		$helper->createParameterFile("$currentdir/s.params",\%resource);

		#call the eetco kernel and keep the output
		$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/x_$x_value");
	
		#get the result value
		my $ptr_res=$helper->parserResultKernel("$currentdir/output/x_$x_value");
		my %res=%{$ptr_res};
		my $current=$res{"$output_name"};
		
		my $normalized_current=$current/$ref;
		
		print(OUTPUTFILE "$x \t $normalized_current \n");
}

close(OUTPUTFILE);

#keep the output file in the current directory
copy("$currentdir/output/$filename.txt","$currentdir/$filename.txt") or die "Copy failed: $!";

####################################################
# gnulpot call
####################################################
if($gnuplot==1)
{
	my @curves_names=("$curve_name");
	$helper->makeGnuplotCurves2D("$currentdir/$filename.txt","$currentdir","$filename","$x_axis_name","$y_axis_name",\@curves_names);
}

####################################################
# clean
####################################################
$helper->deleteFile("$currentdir/dc.params");
$helper->deleteFile("$currentdir/s.params");
$helper->deleteFileInDir("$currentdir/output/");

